"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/monticello/tonel' }"

"{ NameSpace: Smalltalk }"

STONWriter subclass:#TonelSTONWriter
	instanceVariableNames:'aliases'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Tonel-Core'
!

!TonelSTONWriter class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!TonelSTONWriter methodsFor:'accessing'!

aliases
	^ aliases
!

aliases: aDictionary
	aliases := aDictionary
!

stonNameFor: aClass
	^ self aliases 
		at: aClass 
		ifAbsent: [ aClass stonName ]
! !

!TonelSTONWriter methodsFor:'initialization'!

initialize
        super initialize.
        self prettyPrint: true.
        aliases := Dictionary with: OrderedDictionary -> nil

    "Modified: / 07-07-2020 / 11:44:14 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelSTONWriter methodsFor:'private'!

isSimpleSymbol: symbol
	"Customize STON to only consider very clean symbols as literal, 
	for all others err on the safe side and quote them."
	
	symbol isEmpty ifTrue: [ ^ false ].
	('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ' includes: symbol first)
		ifFalse: [ ^ false ].
	^ symbol allSatisfy: [ :each | 
		'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789' includes: each ]
! !

!TonelSTONWriter methodsFor:'writing'!

encodeMap: pairs
	| first |
	first := true.
	writeStream nextPut: ${.
	pairs isEmpty
		ifTrue: [
			self prettyPrintSpace ]
		ifFalse: [
			self indentedDo: [
				pairs size = 1 
					ifTrue: [ self prettyPrintSpace ]
					ifFalse: [ self newlineIndent ].
				pairs keysAndValuesDo: [ :key :value |
					first 
						ifTrue: [ first := false ] 
						ifFalse: [ self mapElementSeparator ].
					self encodeKey: key value: value ] ].
				pairs size = 1 
					ifTrue: [ self prettyPrintSpace ]
					ifFalse: [ self newlineIndent ] ].
	writeStream nextPut: $}
!

writeObject: anObject do: block
        (jsonMode and: [ anObject class ~= STON listClass and: [ anObject class ~= STON mapClass and: [ anObject class ~= OrderedDictionary ] ] ])
                ifTrue: [ self error: 'wrong object class for JSON mode' ].
        self with: anObject do: [
                | stonName |

                (stonName := self stonNameFor: anObject class) notNil ifTrue: [  
                        writeStream nextPutAll: stonName.
                        self prettyPrintSpace ].
                block value ]

    "Modified: / 05-08-2020 / 16:21:18 / Jan Vrany <jan.vrany@labware.com>"
! !

